uniform sampler2D u_texture;
uniform sampler2D u_texture1; // normal map

uniform vec4 u_directionalLightColor;
uniform vec4 u_ambientLightColor;
uniform float u_materialShininess;

varying highp vec2 v_texCoord;
varying highp vec3 v_eyeDirTangentSpace;
varying highp vec3 v_lightDirTangentSpace;

void main() {

	vec3 normalTangentSpace = normalize(texture2D( u_texture1, vec2(v_texCoord.x,v_texCoord.y) ).rgb*2.0 - 1.0); // texture normal in tangent space. in 2D ... we can avoid TBN multiplication and lighting calculation in texture space.
	
	vec4 textureColor = texture2D(u_texture, v_texCoord);
	
	// Direction of the light (from the fragment to the light)
	vec3 l = normalize(v_lightDirTangentSpace);
	
	highp float lambertian = max( dot( normalTangentSpace, l ), 0.0);
	
	// Eye vector (towards the camera)
	vec3 E = normalize(v_eyeDirTangentSpace);

	// Blinn-phong model
	vec3 H = normalize(l + v_eyeDirTangentSpace);
	highp float cosAlpha = max(dot(H, normalTangentSpace), 0.0);
	float shininess = u_materialShininess * 4.0;
	
	vec4 lightAmbientColor = vec4(u_ambientLightColor.rgb * u_ambientLightColor.a, 1.0);
	vec4 ambientColor = textureColor * lightAmbientColor; // vec4(u_ambientLightColor.rgb, textureColor.a);
	vec4 diffuseColor = vec4(textureColor.rgb * lambertian, textureColor.a);

	vec4 specularColor = u_directionalLightColor * pow(cosAlpha, shininess);	

	gl_FragColor = ambientColor + diffuseColor + specularColor;
}