attribute highp vec4 a_position;
attribute highp vec3 a_normal;
attribute highp vec3 a_tangent;
attribute highp vec3 a_binormal;
attribute mediump vec2 a_texCoord;

uniform highp mat4 u_modelViewProjectionMatrix;
uniform highp mat4 u_modelMatrix;
uniform highp mat4 u_directionalLightMatrix;

varying highp vec2 v_texCoord;
varying highp vec3 v_eyeDirTangentSpace;
varying highp vec3 v_lightDirTangentSpace;

highp mat3 transpose3x3(in highp mat3 inMatrix) {
    highp vec3 i0 = inMatrix[0];
    highp vec3 i1 = inMatrix[1];
    highp vec3 i2 = inMatrix[2];

    highp mat3 outMatrix = mat3(
                 vec3(i0.x, i1.x, i2.x),
                 vec3(i0.y, i1.y, i2.y),
                 vec3(i0.z, i1.z, i2.z)
                 );

    return outMatrix;
}

void main() {
	gl_Position = u_modelViewProjectionMatrix * a_position;		
	vec3 vertexPositionWorldSpace = (u_modelMatrix * a_position).xyz;	

	v_texCoord = a_texCoord;
		
	mat3 Model3x3Matrix = mat3(u_modelMatrix[0].xyz, u_modelMatrix[1].xyz, u_modelMatrix[2].xyz);
		
	vec3 normalWorldspace = Model3x3Matrix * a_normal;
	vec3 tangentWorldspace = Model3x3Matrix * a_tangent;
	vec3 bitangentWorldspace = Model3x3Matrix * a_binormal;	

	mat3 TBN = mat3(tangentWorldspace, bitangentWorldspace, normalWorldspace);
	TBN = transpose3x3(TBN);
		
	vec3 lightDirWorld = u_directionalLightMatrix[2].xyz;
	lightDirWorld = normalize(lightDirWorld);
	
	vec3 eyeDirWorldSpace = vec3(0.0, 0.0, 1.0); //always top

	v_eyeDirTangentSpace = TBN * eyeDirWorldSpace;
	v_lightDirTangentSpace = TBN * lightDirWorld;
}

